{% extends "main::container.html" %}

{%block article%}
<article>
    <header>
        <h1>Welcome to Nemo !</h1>
    </header>
    <section>
        <p>Nemo is an extension for Flask and provides a User Interface layer for a CTS Repository</p>
    </section>
    <footer>

    </footer>
</article>
{%endblock%}