<header>
    <span class="content">Text Collections</span>
</header>
<ul class="menu">
    {% for c in main_collections %}
        <li><a href="{{url_for('.r_collection_semantic', objectId=c.id, semantic=c.label|slugify)}}">{{c.label}}</a></li>
    {% endfor %}
</ul>
{% if collections and collections.parents %}
<header>
    <span class="content">Parents</span>
</header>
<ul class="menu">
    {% for parent in collections.parents %}
        <li><a href="{{url_for('.r_collection_semantic', objectId=parent.id, semantic=parent.label|slugify)}}">{{parent.label}}</a></li>
    {% endfor %}
</ul>
{% endif %}