<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <!-- The above 3 meta tags *must* come first in the head; any other head content must come *after* these tags -->
    <title>Capitains Nemo</title>
    {% block metadata %}
      {% include "main::metadata.html" %}
    {% endblock %}

    <!-- Bootstrap --><!-- Latest compiled and minified CSS -->
    <!--<link rel="stylesheet" href="{{url_for('.static', filename='css/teibp.min.css')}}">-->
    <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/css/bootstrap.min.css">
    <link rel="stylesheet" href="{{url_for('.static', filename='css/theme.min.css')}}">
    {% for filename, directory in assets["css"].items() %}
      {% if directory %}<link rel="stylesheet" href="{{url_for('.secondary_assets', filetype='css', asset=filename)}}">
      {% else %}<link rel="stylesheet" href="{{ filename }}">
      {% endif %}
    {% endfor %}
    <!-- HTML5 shim and Respond.js for IE8 support of HTML5 elements and media queries -->
    <!-- WARNING: Respond.js doesn't work if you view the page via file:// -->
    <!--[if lt IE 9]>
      <script src="https://oss.maxcdn.com/html5shiv/3.7.2/html5shiv.min.js"></script>
      <script src="https://oss.maxcdn.com/respond/1.4.2/respond.min.js"></script>
    <![endif]-->
  </head>
  <body>
      <div class="container-fluid">
      <div class="row">
        <nav class="col-sm-2 menus">
          {% include "main::logo.html" %}
        </nav>
        <div class="col-sm-10 right-columns">
              {% include "main::breadcrumb.html" %}
        </div>
      </div>
      <div class="row">
        <nav class="col-sm-2 menus" id="menu-container">
          {% include "main::menu.html" %}
        </nav>
        <div class="col-sm-10 hfeed right-columns" id="reading-container">
          <div class="">
              {% block article %}
              {% endblock %}
          </div>
        </div>
      </div>
      </div>

    <!-- jQuery (necessary for Bootstrap's JavaScript plugins) -->
    <script src="https://ajax.googleapis.com/ajax/libs/jquery/1.11.3/jquery.min.js"></script>
    <!-- Latest compiled and minified JavaScript -->
    <script src="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/js/bootstrap.min.js"></script>
    {% for filename, directory in assets["js"].items() %}
      {% if directory %}<script src="{{url_for('.secondary_assets', filetype='js', asset=filename)}}"></script>
      {% else %}<script src="{{ filename }}"></script>
      {% endif %}
    {% endfor %}

  {% block additionalscript %}
  {% endblock %}
  {% block footer %}
          {% include "main::footer.html" %}
  {% endblock %}
  </body>
</html>
